Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Printing

Public Class frmPrintCover

    Private WithEvents myPrintDocument As New PrintDocument
    Private myContent As CoverContent

    Public Function RepresentDialouge(ByVal Content As CoverContent) As DialogResult

        'Expose passed values to the whole class instance
        myContent = Content

        'This object we'll need for previewing and printing.
        'The content is rendered within this PrintDocument instance.
        '(It doesn't matter if the content is previewed or printed - we use this object in both cases.)
        myPrintDocument = New PrintDocument

        'We set the layout to landscape, otherwise the cover wouldn't fit the page.
        myPrintDocument.DefaultPageSettings.Landscape = True

        'Render output:
        'We assign the PrintDocument to the Preview control.
        'As soon as this happens, the PrintPage event of the PrintDocument is raised,
        'for each page to print one time.
        PrintPreviewControl1.Document = myPrintDocument

        'Show this dialog modally. At this point, the program waits - so to say - 
        'while performing its own event queue until the dialog is closed.
        'Then it returns back to the caller.
        Me.ShowDialog()
    End Function

    Private Sub btnPrintOnDefaultPrinter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrintOnDefaultPrinter.Click

        'Let the user pick the printer:
        Dim locPD As New PrintDialog()
        Dim locDR As DialogResult = locPD.ShowDialog()
        If locDR = Windows.Forms.DialogResult.Cancel Then
            Return
        End If

        'Start printing. As with the preview, for each page to print
        'one correlating PrintPage event is raised.
        myPrintDocument = New PrintDocument
        myPrintDocument.PrinterSettings = locPD.PrinterSettings
        myPrintDocument.DefaultPageSettings.Landscape = True
        myPrintDocument.Print()
    End Sub

    Private Sub myPrintDocument_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles myPrintDocument.PrintPage

        Dim g As Graphics = e.Graphics
        g.PageUnit = GraphicsUnit.Millimeter

        'Define the fonts:
        'TODO: You could extend the dialog with a font picker.
        Dim locFontMainTitle As New Font(FontFamily.GenericSansSerif, 48, FontStyle.Bold)
        Dim locFontSubTitle As New Font(FontFamily.GenericSansSerif, 24, FontStyle.Bold)
        Dim locFontSpine As New Font(FontFamily.GenericSansSerif, 18, FontStyle.Bold)
        Dim locFontFlietext As New Font(FontFamily.GenericSerif, 18, FontStyle.Regular)

        'Set up margins.
        Dim locOffset_X As Single = 10.0F  ' 10 mm rechts und
        Dim locOffset_Y As Single = 10.0F  ' 10 mm oben

        'Paint the left page (back cover).

        'To that end we firstly need as 0.25 mm Pen, which paints in black.
        Dim locPen As New Pen(Color.Black, 0.25F)

        'Paint first rectangle with 13 x 18,3 cm.
        g.DrawRectangle(locPen, locOffset_X, locOffset_Y, 130.0F, 183.0F)

        'Paint the middle rectangle (the gutter).
        g.DrawRectangle(locPen, locOffset_X + 130.0F, locOffset_Y, 13.0F, 183.0F)

        'And the third one, which is the front page.
        g.DrawRectangle(locPen, locOffset_X + 130.0F + 13.0F, locOffset_Y, 130.0F, 183.0F)

        'We have to draw the back page's description text. For that, firstly, we
        'need a StringFormat Object, which sets the alignment.
        Dim locStringFormat As StringFormat = DirectCast(StringFormat.GenericTypographic.Clone, StringFormat)

        'Vertically centered (Distance from bottom and top).
        locStringFormat.LineAlignment = StringAlignment.Center

        'Horizontally centered (Distance from left and right side).
        locStringFormat.Alignment = StringAlignment.Center

        'Title text (upper third) in subtitle font, if needed multilined and centered.
        g.DrawString(myContent.MovieTitle, locFontSubTitle, Brushes.Black, _
                New RectangleF(locOffset_X, locOffset_Y, 130.0F, 183.0F / 3.0F), _
                locStringFormat)

        'below, the description (lower two third of the page) centered, on top of the lower margin.
        '(since we write from left to right, "near" is left aligned).
        locStringFormat.LineAlignment = StringAlignment.Far
        g.DrawString(myContent.Description, locFontFlietext, Brushes.Black, _
                New RectangleF(locOffset_X, locOffset_Y + 183.0F / 3.0F, 130.0F, 183.0F / 2.0F), _
                locStringFormat)

        'And now: the main title including the actors.

        'For that we firstly draw, if provided, the cover image...
        If myContent.CoverImage IsNot Nothing Then
            With myContent.CoverImage
                g.DrawImage(myContent.CoverImage, _
                    New RectangleF(locOffset_X + 130.0F + 13.0F, locOffset_Y, 130.0F, 183.0F), _
                    New RectangleF(0, 0, .Width, .Height), _
                    GraphicsUnit.Pixel)
            End With
        End If

        '...centered on the front page. For that, we have to measure the margin
        'and then calculate the coordinates to put the pictures so it fits the page in the middle.
        Dim locMainTitleHeight As Single
        Dim locActorHeight As Single

        'This variable we need to measure the height.
        Dim locPrintDimensions As New SizeF(130.0F, 0.0F)

        locPrintDimensions = g.MeasureString(myContent.MovieTitle, locFontMainTitle, locPrintDimensions)
        locMainTitleHeight = locPrintDimensions.Height

        'the same counts for the actor description.
        locPrintDimensions = New SizeF(130.0F, 0.0F)
        locPrintDimensions = g.MeasureString(myContent.Actor, locFontSubTitle, locPrintDimensions)
        locActorHeight = locPrintDimensions.Height

        'Render the main title:
        locStringFormat.LineAlignment = StringAlignment.Center
        locStringFormat.Alignment = StringAlignment.Center

        g.DrawString(myContent.MovieTitle, locFontMainTitle, Brushes.LightGray, _
            New RectangleF( _
                locOffset_X + 130.0F + 13.0F, _
                locOffset_Y + (183.0F / 2.0F - (locMainTitleHeight + 10.0F + locActorHeight) / 2.0F), _
                130.0F, _
                locMainTitleHeight), _
            locStringFormat)

        'Authors below:
        g.DrawString(myContent.Actor, locFontSubTitle, Brushes.Black, _
            New RectangleF( _
                locOffset_X + 130.0F + 13.0F, _
                locOffset_Y + (183.0F / 2.0F - (locMainTitleHeight + 10.0F + locActorHeight) / 2.0F) _
                    + 10.0F + locMainTitleHeight, _
                130.0F, _
                locActorHeight), _
            locStringFormat)

        'Caption for the gutter, the "spine" of the book.
        'We have to rotate the text by 90 degrees.
        locStringFormat.FormatFlags = StringFormatFlags.DirectionVertical
        g.DrawString(myContent.MovieTitle, locFontSpine, Brushes.Black, _
                New RectangleF(locOffset_X + 130.0F, locOffset_Y, 13.0F, 183.0F), _
                locStringFormat)

        '"...and that, Gentlemen, is how we do that!" - Tom Hanks as Jim Lovell, after S4B cutoff in Apollo 13

    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub
End Class