Public Class AlarmTrigger

    Private WithEvents myTrigger As Timer
    Private myAlarmText As String
    Private myTimeOfAlarm As Date
    Private myIsAlarmActivated As Boolean
    Private myThreashold As Integer = 2

    ''' <summary>
    ''' Raised, if a certain time is reached.
    ''' </summary>
    ''' <param name="Sender">Object, which raised this event.</param>
    ''' <param name="e">AlarmEventArgs containing further information about the event.</param>
    ''' <remarks></remarks>
    Public Event Alarm(ByVal Sender As Object, ByVal e As AlarmEventArgs)

    Sub New(ByVal timeOfAlarm As Date)
        Me.TimeOfAlarm = timeOfAlarm
    End Sub

    Sub New(ByVal timeOfAlarm As Date, ByVal activated As Boolean)
        Me.TimeOfAlarm = timeOfAlarm
        Me.AlarmActivated = activated
    End Sub


    ''' <summary>
    ''' Sets or gets if the alarm is activated (true) or not (false).
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property AlarmActivated() As Boolean
        Get
            Return myIsAlarmActivated
        End Get
        Set(ByVal value As Boolean)
            If value <> myIsAlarmActivated Then
                If value Then
                    myTrigger = New Timer()
                    myTrigger.Interval = 1000
                    myTrigger.Start()
                    myIsAlarmActivated = True
                Else
                    myTrigger.Dispose()
                End If
            End If

        End Set
    End Property

    ''' <summary>
    ''' Sets or gets at which time the alarm - activation presumed - will go off.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property TimeOfAlarm() As Date
        Get
            Return myTimeOfAlarm
        End Get
        Set(ByVal value As Date)
            'Prevent the alarm time to be in the past. 
            value = Date.Now.Date.Add(value.TimeOfDay)
            If value < Date.Now Then
                value = value.AddDays(1)
            End If
            myTimeOfAlarm = value
        End Set
    End Property

    Private Sub myTrigger_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles myTrigger.Tick
        If myTimeOfAlarm < DateTime.Now Then
            Dim locAlarmTimeEventArgs As New AlarmEventArgs(TimeOfAlarm)
            OnAlarm(locAlarmTimeEventArgs)
            If locAlarmTimeEventArgs.Reset Then
                TimeOfAlarm = locAlarmTimeEventArgs.TimeOfAlarm
            Else
                AlarmActivated = False
            End If
        End If
    End Sub

    ''' <summary>
    ''' Raises the Alarm Event.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Sub OnAlarm(ByVal e As AlarmEventArgs)
        RaiseEvent Alarm(Me, e)
    End Sub
End Class

Public Class AlarmEventArgs
    Inherits EventArgs

    Sub New(ByVal timeOfAlarm As Date)
        timeOfAlarm = timeOfAlarm
        Reset = True
    End Sub

    Sub New(ByVal timeOfAlarm As Date, ByVal reset As Boolean)
        timeOfAlarm = timeOfAlarm
        Reset = reset
    End Sub

    Public Property TimeOfAlarm() As Date
    Public Property Reset() As Boolean
End Class

