Class DynamicList(Of flexibleDataType)
    Implements IEnumerable

    Private myStepIncreaser As Integer
    Private myCurrentArraySize As Integer
    Private myCurrentCounter As Integer
    Private myArray() As Object

    Sub New()

        MyClass.New(16)

    End Sub

    Sub New(ByVal StepIncreaser As Integer)

        myStepIncreaser = StepIncreaser
        myCurrentArraySize = myStepIncreaser
        ReDim myArray(myCurrentArraySize)

    End Sub

    Sub Add(ByVal Item As flexibleDataType)

        'Check whether the current array limit has been reached
        If myCurrentCounter = myCurrentArraySize - 1 Then

            'Create new array with more memory,
            'and copy elemente into it.
            'New array becomes larger:
            myCurrentArraySize += myStepIncreaser

            'Create temporary array
            Dim locTempArray(myCurrentArraySize - 1) As Object

            'Copy elements
            'Important: Unlike in VB6, you have to take
            'care of the copying yourself!
            Array.Copy(myArray, locTempArray, myArray.Length)

            'Assign the temporary array to the member array
            myArray = locTempArray

            'Next time more elements are reserved!
            myStepIncreaser *= 2
        End If

        'Save the element in the array
        myArray(myCurrentCounter) = Item

        'Increase the pointer to the next element
        myCurrentCounter += 1
    End Sub

    'Returns the number of the existing elements
    Public Overridable ReadOnly Property Count() As Integer
        Get
            Return myCurrentCounter
        End Get
    End Property

    'Permits assignments and queries
    Default Public Overridable Property Item(ByVal Index As Integer) As flexibleDataType
        Get
            Return CType(myArray(Index), flexibleDataType)
        End Get

        Set(ByVal Value As flexibleDataType)
            myArray(Index) = Value
        End Set
    End Property

    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        'GetEnumerator has to be implemented due to IEnumerable.
        'IEnumerable we need again to support For/Each.

        'An Array supports IEnumerable by default, that's
        'why we're "borrowing" the GetEnumerator-
        'Functionality from the Array.

        'There is one problem, though: Although we already 
        'have an array - we cannot use it, since it
        'contains most of the time more elements than we
        'are actually using in the collection.

        'So, we are creating a temporary array with
        'the exact same count of items we use in this collection.
        'This is lazy and not high-performance, but it works. :-)
        Dim locTempArray(myCurrentCounter - 1) As Object
        Array.Copy(myArray, locTempArray, myCurrentCounter)
        Return locTempArray.GetEnumerator
    End Function
End Class
