Imports System.IO

<Serializable()>
Public Class Address

    Private myDateCreated As DateTime
    Private myCreatedBy As String

    Sub New(ByVal FirstName As String, ByVal LastName As String, ByVal Street As String, ByVal ZipCity As String)
        'Constructor creates all backing fields.
        Me.LastName = LastName
        Me.FirstName = FirstName
        Me.Street = Street
        Me.ZipCity = ZipCity
        myDateCreated = DateTime.Now
        myCreatedBy = My.User.Name
    End Sub

    'Write public fields to the file - in Soap-Format
    Public Shared Sub SerializeSoapToFile(ByVal adr As Address, ByVal Filename As String)
        'In Addition to the book's printed sample, 
        'let's check, if the folder exist and if not, create it.
        Dim locFileInfo As New FileInfo(Filename)

        'We can easly access the padth part of the full filename
        'with the directory property, and check, if it exists
        'with the exist method.
        If Not locFileInfo.Directory.Exists Then
            'If it does not exist, we create it - it's that simple! :-)
            locFileInfo.Directory.Create()
        End If

        SoapSerializer.SerializeToFile(New FileInfo(Filename), adr)
    End Sub

    'Read from file and create new address object - Soap-Format
    Public Shared Function SerializeSoapFromFile(ByVal Filename As String) As Address
        Return DirectCast(SoapSerializer.DeserializeFromFile(New FileInfo(Filename)), Address)
    End Function

    'Write public fields to the file - Binary-Format
    Public Shared Sub SerializeBinToFile(ByVal adr As Address, ByVal Filename As String)
        'In Addition to the book's printed sample, 
        'let's check, if the folder exist and if not, create it.
        Dim locFileInfo As New FileInfo(Filename)

        'We can easly access the padth part of the full filename
        'with the directory property, and check, if it exists
        'with the exist method.
        If Not locFileInfo.Directory.Exists Then
            'If it does not exist, we create it - it's that simple! :-)
            locFileInfo.Directory.Create()
        End If

        BinarySerializer.SerializeToFile(New FileInfo(Filename), adr)
    End Sub

    'Read from file and create new address object - Binary-Format
    Public Shared Function SerializeBinFromFile(ByVal Filename As String) As Address
        Return DirectCast(BinarySerializer.DeserializeFromFile(New FileInfo(Filename)), Address)
    End Function

    'Read the stream of data from the file and create a new address object out of that.
    Public Shared Function SerializeFromFile(ByVal Filename As String) As Address
        Dim locStreamReader As New StreamReader(Filename, System.Text.Encoding.Default)
        Dim locAddress As Address
        With locStreamReader
            locAddress = New Address(.ReadLine, .ReadLine, .ReadLine, .ReadLine)
        End With
        locStreamReader.Close()
        Return locAddress
    End Function

    'Auto-implemented properties.
    Public Property LastName() As String
    Public Property FirstName() As String
    Public Property Street() As String
    Public Property ZipCity() As String

    'The following properties are read only, because the developer 
    'shouldn't be able to manipulate this date.
    Public ReadOnly Property DateCreated() As DateTime
        Get
            Return myDateCreated
        End Get
    End Property

    Public ReadOnly Property CreatedBy() As String
        Get
            Return myCreatedBy
        End Get
    End Property


End Class
