Imports System.Reflection
Imports System.IO

Module mdlMain

    Sub Main()

        Dim locAddress As New Address("Adriana", "Ardelean", "Book Ave. 100", "De-32154 Publisher Valley") With
            {.IsFriendWith = New List(Of Address) From
                {New Address("Klaus", "Lffelmann", "One Microsoft Way", "WA 99999 Redmond")}}

        PrintObjectInfo(locAddress)
        Console.ReadLine()
        'Exit Sub

        'A few Type explorations:
        Dim locFriend As New Address("Adriana", "Ardelean", "Book Ave. 100", "De-32154 Publish Valley")

        Dim locTest As New Address("Klaus", "Lffelmann", "17 Vacation Road", "De-59555 Illbeback")
        Dim locType1, locType2 As Type
        locType1 = locTest.GetType

        'Important: TypeOf works only together with the Is operator:
        If TypeOf locTest Is Reflection01.Address Then
            Console.WriteLine("Address type recognized!")
        Else
            Console.WriteLine("Address type not recognized!")
        End If

        'And this is also an alternative with does the same:
        If locTest.GetType Is GetType(Reflection01.Address) Then
            Console.WriteLine("Address type recognized!")
        Else
            Console.WriteLine("Address type not recognized!")
        End If

        'This is how it works when two type object 
        'must be compared to each other:

        locType2 = GetType(Address)                     ' Standard: GetType as operator
        locType2 = GetType(Reflection01.Address)        ' Alternative: with namespace
        locType2 = Type.GetType("Reflection01.Address") ' Alternative: from string

        Console.WriteLine("Type " + locType1.FullName + _
                CStr(IIf(locType1 Is locType2, " corresponds to ", " does not correspond to ")) + _
                "Type " + locType2.FullName)

        'Find out whether a type variable (e.g. base class or interface)
        'is compatible to the actual instantiated object. 
        Dim adresseMitGebDatum As New DerivedClass With
            {.LastName = "Leenings", .FirstName = "Ramona", .Street = "Somewhere Street 12",
             .ZipCity = "AU-12345 Vienna",
             .IsFriendWith = New List(Of Address) From {locTest, locFriend},
             .DateOfBirth = #7/24/1991#}

        If GetType(Address).IsAssignableFrom(adresseMitGebDatum.GetType) Then
            Console.WriteLine(adresseMitGebDatum.ToShortString &
                              " can be saved in the type 'Address'.")
        End If

        Console.ReadLine()

    End Sub

    Sub PrintObjectInfo(ByVal [Object] As Object)

        'Find out the object type to access the object content
        Dim locTypeInstanz As Type = [Object].GetType

        'Return the attributes that are not custom:
        Console.WriteLine("Attributes of class:" + locTypeInstanz.FullName)
        Console.WriteLine("Standard attributes:")
        Console.WriteLine("    *" + locTypeInstanz.Attributes.ToString())
        Console.WriteLine()

        'Find out members and their optional attributes
        Dim locMembers() As MemberInfo

        'Gets everything (except for non-public elements); 
        'locMembers = locTypeInstanz.GetMembers()

        'Makes sure that only non-public fields are retrieved:
        locMembers = locTypeInstanz.GetMembers(BindingFlags.Instance Or BindingFlags.NonPublic)
        Console.WriteLine("List of Members:")
        For Each locMember As MemberInfo In locMembers
            Console.WriteLine("    *" + locMember.Name + ", " _
                                + locMember.MemberType.ToString)
            If locMember.GetCustomAttributes(True).Length > 0 Then
                Console.WriteLine("     " + New String("-"c, locMember.Name.Length))
            End If

            'If we are dealing with properties, we are also retrieving the value.
            If locMember.MemberType = MemberTypes.Property Then
                Dim locPropertyInfo As PropertyInfo = DirectCast(locMember, PropertyInfo)
                Dim value = locPropertyInfo.GetValue([Object], Nothing)
                If value IsNot Nothing Then
                    Console.WriteLine("     Value: " + value.ToString)
                End If
            End If

            'Return field information when the appropriate
            'Bindingflags were specified.
            If locMember.MemberType = MemberTypes.Field Then
                Dim locFieldInfo As FieldInfo = DirectCast(locMember, FieldInfo)
                Dim value = locFieldInfo.GetValue([Object])
                If value IsNot Nothing Then
                    Console.WriteLine("     Value: " + value.ToString)
                End If
            End If
        Next
    End Sub

End Module

<Serializable()> _
Public Class Address

    Sub New()
        MyBase.New()
    End Sub

    Private myDateCreated As DateTime
    Private myCreatedBy As String

    Sub New(ByVal FirstName As String, ByVal LastName As String, ByVal Street As String, ByVal ZipCity As String)
        Me.LastName = LastName
        Me.FirstName = FirstName
        Me.Street = Street
        Me.ZipCity = ZipCity
        myDateCreated = DateTime.Now
        myCreatedBy = Environment.UserName
    End Sub

    Public Property LastName() As String
    Public Property FirstName() As String
    Public Property Street() As String
    Public Property ZipCity() As String
    Public Property IsFriendWith() As List(Of Address)

    Public ReadOnly Property DateCreated() As DateTime
        Get
            Return myDateCreated
        End Get
    End Property

    Public ReadOnly Property CreatedBy() As String
        Get
            Return myCreatedBy
        End Get
    End Property

    Public Overrides Function ToString() As String
        Dim locTemp As String
        locTemp = LastName + ", " + FirstName + ", " + Street + ", " + ZipCity + vbNewLine
        locTemp += "--- Is friend with: ---" + vbNewLine
        For Each locAdr As Address In IsFriendWith
            locTemp += "   * " + locAdr.ToShortString() + vbNewLine
        Next
        locTemp += vbNewLine
        Return locTemp
    End Function

    Public Function ToShortString() As String
        Return LastName + ", " + FirstName
    End Function
End Class

Public Class DerivedClass
    Inherits Address

    Property DateOfBirth As Date

End Class
