﻿Imports System.Threading
Imports System.Threading.Tasks

Module Module1

    Private myThreadCount As Integer
    Private myList As System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry)

    Sub Main()

        'CancelableVersion()
        'Return

        myList = New System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry)
        Dim sw = Stopwatch.StartNew

        Parallel.For(1, 50, Sub(i)
                                myList.Add(New ItemLogEntry With {.TickTime = sw.ElapsedMilliseconds,
                                                                  .value = i,
                                                                  .threadid = Thread.CurrentThread.ManagedThreadId})
                                'Pretend it takes a long time but the
                                'high numbers take a lot more time:
                                Console.Write(".")
                                Thread.SpinWait(i * 200000)
                            End Sub)
        Console.WriteLine()
        Console.WriteLine("Sorted by Thread-ID:")

        Dim sortedById = From item In myList
                     Order By item.ThreadId, item.TickTime

        sortedById.ToList.ForEach(Sub(item)
                                      Console.WriteLine(item.ToString)
                                  End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

        Console.WriteLine()
        Console.WriteLine("Sorted by duration:")

        Dim sortedByTime = From item In myList
                     Order By item.TickTime

        sortedByTime.ToList.ForEach(Sub(item)
                                        Console.WriteLine(item.ToString)
                                    End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

    End Sub

    Public Structure ItemLogEntry
        Public TickTime As Long
        Public ThreadId As Long
        Public Value As Integer

        Public Overrides Function ToString() As String
            Return String.Format("ThreadId {0:0000}: @{1:000000} ms - Value:{2:000000}",
                                 ThreadId, TickTime, Value)
        End Function
    End Structure

    Sub CancelableVersion()

        myList = New System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry)
        Dim sw = Stopwatch.StartNew

        Dim flagWhichControlsThatAllValuesBelowTheCurrentGetProcessesed = False

        Try
            Parallel.For(1, 50,
                         Sub(i As Integer, parLoopState As ParallelLoopState)
                             myList.Add(New ItemLogEntry With {.TickTime = sw.ElapsedMilliseconds,
                                                               .value = i,
                                                               .threadid = Thread.CurrentThread.ManagedThreadId})
                             'Pretend it takes a long time but the
                             'high numbers take a lot more time:
                             Console.Write(".")
                             Thread.SpinWait(i * 200000)

                             'Knock off at 25!
                             If i = 25 Then
                                 If flagWhichControlsThatAllValuesBelowTheCurrentGetProcessesed Then
                                     'Every parallel iteration which processes values below 25 will 
                                     'continue until there are only values left above 25.
                                     parLoopState.Break()
                                 Else
                                     'Alternatively, with stop, all iterations will stop immediately.
                                     parLoopState.Stop()
                                 End If
                                 Console.Write("Exited for!")
                             End If
                         End Sub)
        Catch ex As Exception
            Console.WriteLine("Exited For!")
        End Try



        Console.WriteLine()
        Console.WriteLine("Sorted by Thread-ID:")
        Dim sortedById = From item In myList
                     Order By item.ThreadId, item.TickTime

        sortedById.ToList.ForEach(Sub(item)
                                      Console.WriteLine(item.ToString)
                                  End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

        Console.WriteLine()
        Console.WriteLine("Sorted by Duration:")
        Dim sortedByTime = From item In myList
                     Order By item.TickTime

        sortedByTime.ToList.ForEach(Sub(item)
                                        Console.WriteLine(item.ToString)
                                    End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

    End Sub

End Module
